#!/usr/bin/perl
# linklnk       *** internal script (called by vmake !)
#
# @(#)linklnk		1997-11-18
#
# link pascal object modules to create executable image
#
# TOOL          directory for tools
# TOOL_LIB      specific directory for tool libraries (32bit/64bit specific)
# INSTLIB       specific directory for installed libraries (32bit/64bit specific)
# LDFLAGS       optional loader flags
# KEEPFLG       optional keep intermediate files flag
#
# 1996-10-16 G. Gromann: -lib=ptocnt[s|r] in Descriptions setzt @LIBPC explizit neu
# 			  mit $TOOL/lib/ptocnt[s|r].lib
#
# 1997-02-04 GG: Map-Files werden nach $SYS/map/$VMAKE_VERSION verschoben
# 1998-09-09 &gar: MAP-Files moved to $DBROOT/Support/Mapfiles -> 2 files:
#                  normal (xxx.map) and one with second extension <VMAKE_EXT>
#
# 2000-10-12 JoergMensing $TOOL/lib replaced by $TOOL_LIB, $INSTROOT/lib by $INSTLIB
#
#
#    ========== licence begin  GPL
#    Copyright (C) 2001 SAP AG
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of the GNU General Public License
#    as published by the Free Software Foundation; either version 2
#    of the License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#    ========== licence end
#

package linklnk;
use Env;
use File::Copy;
use ToolLib;

# Get LDFLAGS, LD, LIB*
do "$TOOLVARS";
if ( $@ ) { print "$@"; exit }
ToolTrace("linklnk called with @ARGV\n");

$USAGE = "usage: linklnk [-p] [ld-flags] objects\n";

$DEBUG  = "";
$PROF   = "";
$SCRIPT = "";
$RCFILE = "";
$NOTHREAD = 0; # CTS 1110865
local @LDCMD=();
@BuildModules=();
@SLIBS= ();
$MAP    = "-map";
$NOWHATSTRING = 0;

$OUTFILE = "a.out";
$PROG = $OUTFILE;   # some systems forbid some extensions for executables

do "$TOOL/bin/link_opts$TOOLEXT";
if ( $@ ) { print "$@"; exit }


while ($_ = $ARGV[0], /^-/) {
    shift;
    last if /^--$/;

    if ( /^-linkoption=(.+)$/ ) { push @OFLAGS, $1; next }

    if ( /^-tooloption=(.+)$/ ) { next }

    if (/^-.*/) { push @OFLAGS, $_; next }
 } #while

if ( @ARGV < 1 ) { print $USAGE;  exit 1; }

if (!$DEBUG)
    { $STRIPDEBUG = "y" }

if  ( ! defined($ARCH_TYPE))
{
    print "Warning: (linklnk) ARCH_TYPE not set !\n";
    $ARCH_TYPE = ".lib";
}


# PTS 1103511
$ARCH_SEARCH_STRING="\\".$ARCH_TYPE;
@OBJECTS = ();
foreach $F (@ARGV) {
    if ($F =~ /(.*)\.dll(\..*)/) {
       $SUFF = $2;
       push @OBJECTS, "$1.$ARCH_TYPE$SUFF" ;
       next;
    }
    if ($F =~ /(.*)$ARCH_SEARCH_STRING(\..*)/) { $F="$1$ARCH_TYPE";}
	push @OBJECTS, $F;
} #foraech


push @LDCMD, $LD;
if ($LD eq $CXX) {
	push @LDCMD, @LDXXFLAGS;
} else {
	push @LDCMD, @LDFLAGS;
}

push @LDCMD, @THREAD_LDFLAGS;  # CTS 1110865
push @LDCMD, @LDEXEFLAGS, @OFLAGS;


$PROG =~ /(.*)\.exe/, $BASE = $1 ne "" ? $1 : $PROG;

if ( $SCRIPT ne "" ) {
   print "SCRIPT = $SCRIPT $BASE\n";
   system("$TOOL/bin/$SCRIPT $BASE") == 0
		     or die "Error while executing \"$TOOL/bin/$SCRIPT $BASE\"\n".
			 "message: $!\n";
}

if ( $RCFILE ne "" ) {
	@ARGV = ( "$BASE", "$RCFILE", @RCINCL );
	do "$TOOL/bin/comprc$TOOLEXT";
	if ( $@ ) { warn "Error while executing comprc:\n", "$@"; exit }
}

if ( $CURR_VERSION !~ /^f/i ) { $DEBUG = "y" }

if ( $DEBUG && $PROF ) {
        push @LDCMD, "-debug", "-debugtype:$LD_PROFDEBGTYPE", "/PROFILE"; }
else {
   if ( $PROF ) {
      push @LDCMD, "-debug", "-debugtype:$LD_PROFDEBGTYPE", "/PROFILE";
   }
   elsif ( ! $NOLINKDEBUG ) {
      push @LDCMD, "-debug", "-debugtype:cv", "/OPT:REF";
   };
}
# PTS: 1107332
if (($ENV{_MSC_VER} lt "13") && ($OWN !~ /_SRC/))
{
   push @LDCMD, "-pdb:none";
}
push @BuildModules, ToolLib::CreateBuildNrModule(@KEEPFLG,$DEBUG?("-g"):(),$NOTHREAD?("-nothread"):());

push @LDCMD, ( "-out:$PROG" , $MAP );
push @LDCMD, @OBJECTS;
push @LDCMD, @Lib unless ($ENV{APOMAKE});
push @LDCMD, @BuildModules;
push @LDCMD, @SLIBS;
push @LDCMD, @LIBPC;
push @LDCMD, @LIBSCR;
push @LDCMD, @LIBMATH;
push @LDCMD, @LIBLD;    # CTS 1106257

push @LDCMD, @DYNAMIC_LIB;
push @LDCMD, @LIBNET;
push @LDCMD, @LIBMPROC;
push @LDCMD, ( $PROF );
push @LDCMD, @LIBPOGO;

$LINKCMD=join " ", @LDCMD;
$LINKCMD=~s/\s(\S*)\*(\S*)\s/ "$1\*$2" /g;
$LINKCMD=~s/\s(\S*)\?(\S*)\s/ "$1\?$2" /g;
$LINKCMD.=" ".ToolLib::getWhatModule($PROG, $LINKCMD, @KEEPFLG,$DEBUG?("-g"):(),$NOTHREAD?("-nothread"):())
		unless ($ENV{APOMAKE} || $NOWHATSTRING );
print ("$LINKCMD\n") if ($ENV{NOQUIET});
ToolTrace("linklnk call: $LINKCMD\n");
open(LINK,"$LINKCMD 2>&1 |");
while(defined($line=<LINK>))
{
		# ignore lines with ignored values
		next if ( ($LNK_IGNORE ne "") && ($line=~/($LNK_IGNORE)/i));
		print $line;
}
close(LINK);
unless ( $? == 0 )
{
		 die "Error while executing $LINKCMD \nmessage: $!\n";
}

if ( -f $PROG ) {

	($VMAKE_EXT) = $CURR_VERSION =~ /^(.)/;
	$VMAKE_EXT = lc($VMAKE_EXT);

  if ($ENV{_MSC_VER} gt "12") {
		if ( ! -d "$OWN/usr/symbols" ) { mkdir("$OWN/usr/symbols", 0) }
		copy "$BASE.pdb","$OWN/usr/symbols/$BASE.pdb.$VMAKE_EXT"
       || warn "Can't copy $BASE.pdb to $OWN/usr/symbols/$BASE.pdb.$VMAKE_EXT: $!\n";
		unlink "$OWN/usr/symbols/$BASE.pdb";
		rename "$BASE.pdb", "$OWN/usr/symbols/$BASE.pdb"
       || warn "Can't rename $BASE.pdb to $OWN/usr/symbols/$BASE.pdb: $!\n";
  } else {
		if ( $STRIPDEBUG && !$PROF ) {
         ToolTrace("separate debug info from $PROG\n");
         ToolTrace("linklnk called:\symedit X -qro $BASE.DBG $PROG\n");
		   system("$TOOL/pgm/symedit X -qro $BASE.DBG $PROG") == 0
				 or die "Error while executing \"$TOOL/pgm/symedit X -qro $BASE.DBG $PROG\"\n".
				 "message: $!\n";
		   if ( ! -d "$OWN/usr/symbols" ) { mkdir("$OWN/usr/symbols", 0) }
		   copy "$BASE.DBG","$OWN/usr/symbols/$BASE.dbg.$VMAKE_EXT"
          || warn "Can't copy $BASE.dbg to $OWN/usr/symbols/$BASE.dbg.$VMAKE_EXT: $!\n";
		   unlink "$OWN/usr/symbols/$BASE.dbg";
		   rename "$BASE.DBG", "$OWN/usr/symbols/$BASE.dbg"
          || warn "Can't rename $BASE.dbg to $OWN/usr/symbols/$BASE.dbg: $!\n";
		}
		else {
		   unlink "$OWN/usr/symbols/$BASE.dbg.$VMAKE_EXT";
		   unlink "$OWN/usr/symbols/$BASE.dbg";
		}
  }
}

if ( @KEEPFLG == 0 ) {
   unlink "$BASE.res"; # resource pre-object
   unlink "$BASE.rbj"; # resource object
   unlink <*.pch>; # precompiled header
   if ( -f $PROG && -f "$BASE.map" ) {
      mkdir ("$OWN/usr/Support", 0775) if ( ! -d "$OWN/usr/Support") ;
	  mkdir ("$OWN/usr/Support/Mapfiles", 0775) if ( ! -d "$OWN/usr/Support/Mapfiles") ;
	  copy "$BASE.map", "$OWN/usr/Support/Mapfiles/$BASE.map.$VMAKE_EXT"
	  || warn "Can't copy $BASE.map to $OWN/usr/Support/Mapfiles/$BASE.map.$VMAKE_EXT: $!\n";
	  unlink "$OWN/usr/Support/Mapfiles/$BASE.map";
      rename "$BASE.map", "$OWN/usr/Support/Mapfiles/$BASE.map";
   }
}

if ( $OUTFILE ne $PROG ) { unlink $OUTFILE; rename($PROG, $OUTFILE) }


